import React, { useState, useEffect, useRef, useContext, useCallback } from 'react';
import PropTypes from 'prop-types';
import ReactDOM from 'react-dom';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function ModalPortal(_ref) {
  var children = _ref.children,
      isOpen = _ref.isOpen;

  var _useState = useState(),
      _useState2 = _slicedToArray(_useState, 2),
      container = _useState2[0],
      setContainer = _useState2[1];

  useEffect(function () {
    var body = document.getElementsByTagName('body')[0];

    if (!container) {
      var containerElem = document.createElement('div');
      body.appendChild(containerElem);
      setContainer(containerElem);
    }

    return function () {
      body.removeChild(container);
    };
  }, []); //FIXME: prop to define if modal will be always included into DOM

  if (!container || !isOpen) {
    return '';
  }

  return ReactDOM.createPortal(React.createElement(React.Fragment, null, children, isOpen && React.createElement("div", {
    className: "modal-backdrop fade show"
  })), container);
}
ModalPortal.propTypes = {
  children: PropTypes.any
};

function safeClick(callback, callbackArgs) {
  return function (e) {
    stopPropagation(e);
    e.preventDefault();
    callback(callbackArgs);
  };
}
function awaitForAsyncTask(task, callback) {
  return function () {
    var res = task();

    if (!res || !res.then) {
      callback();
      return;
    }

    res.then(callback);
  };
}
function stopPropagation(e) {
  e.stopPropagation();
}

var ESCAPE_KEYCODE = 27;
function Modal(_ref) {
  var title = _ref.title,
      body = _ref.body,
      onClose = _ref.onClose,
      isOpen = _ref.isOpen,
      footer = _ref.footer,
      staticBackdrop = _ref.staticBackdrop,
      scrollable = _ref.scrollable,
      centered = _ref.centered,
      size = _ref.size,
      keyboard = _ref.keyboard;
  var modalRef = useRef(null);

  function closeIfEscape(event) {
    if (keyboard && event.which === ESCAPE_KEYCODE) {
      onClose();
    }
  }

  useEffect(function () {
    modalRef.current.addEventListener('keydown', closeIfEscape);
    return function () {
      modalRef.current.removeEventListener('keydown', closeIfEscape);
    };
  }, [keyboard]);
  useEffect(function () {
    var body = document.querySelector('body');

    if (isOpen) {
      body.classList.add('modal-open');
      modalRef.current.style.display = 'block';
      modalRef.current.classList.add('show');
      modalRef.current.focus();
    } else {
      body.classList.remove('modal-open');
      modalRef.current.style.display = 'none';
      modalRef.current.classList.remove('show');
    }
  }, [isOpen]);
  return React.createElement("div", {
    className: "modal fade",
    tabIndex: "-1",
    role: "dialog",
    ref: modalRef,
    onClick: function onClick(e) {
      e.stopPropagation();

      if (!staticBackdrop) {
        onClose();
      }
    }
  }, React.createElement("div", {
    className: "modal-dialog ".concat(scrollable ? 'modal-dialog-scrollable' : '', " ").concat(centered ? 'modal-dialog-centered' : '', " ").concat(size ? "modal-".concat(size) : ''),
    role: "document",
    onClick: stopPropagation
  }, React.createElement("div", {
    className: "modal-content"
  }, React.createElement("div", {
    className: "modal-header"
  }, React.createElement("h5", {
    className: "modal-title"
  }, title), React.createElement("button", {
    type: "button",
    className: "close",
    onClick: onClose,
    "aria-label": "Close"
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\xD7"))), React.createElement("div", {
    className: "modal-body"
  }, renderObjectOrFunction(body, {
    close: onClose
  })), footer && React.createElement("div", {
    className: "modal-footer"
  }, renderObjectOrFunction(footer, {
    close: onClose
  })))));
}
Modal.defaultProps = {
  centered: true,
  keyboard: true,
  scrollable: false,
  size: '',
  staticBackdrop: false
};
Modal.propTypes = {
  body: PropTypes.oneOfType([PropTypes.string, PropTypes.element, PropTypes.func]),
  centered: PropTypes.bool,
  footer: PropTypes.oneOfType([PropTypes.string, PropTypes.element, PropTypes.func]),
  isOpen: PropTypes.bool,
  keyboard: PropTypes.bool,
  onClose: PropTypes.func,
  scrollable: PropTypes.bool,
  size: PropTypes.oneOf(['sm', 'lg', 'xl', '']),
  staticBackdrop: PropTypes.bool,
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.element])
};

function renderObjectOrFunction(content, params) {
  if (typeof content === 'function') {
    return content(params);
  }

  return content;
}

function useOpenState() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

  var _useState = useState(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      _isOpen = _useState2[0],
      setIsOpen = _useState2[1];

  return {
    open: function open() {
      setIsOpen(true);
    },
    close: function close() {
      setIsOpen(false);
    },
    isOpen: function isOpen() {
      return _isOpen;
    }
  };
}

function Dialog(_ref) {
  var children = _ref.children,
      props = _objectWithoutProperties(_ref, ["children"]);

  var _useOpenState = useOpenState(),
      isOpen = _useOpenState.isOpen,
      open = _useOpenState.open,
      close = _useOpenState.close;

  return React.createElement(React.Fragment, null, React.createElement(DialogTrigger, {
    open: open
  }, children), React.createElement(ModalPortal, {
    isOpen: isOpen()
  }, React.createElement(Modal, _extends({}, props, {
    onClose: close,
    isOpen: isOpen()
  }))));
}
Dialog.propTypes = {
  content: PropTypes.any,
  children: PropTypes.any
};

function DialogTrigger(_ref2) {
  var children = _ref2.children,
      open = _ref2.open;
  return React.cloneElement(children, {
    onClick: safeClick(open)
  });
}

function AlertDialog(_ref) {
  var title = _ref.title,
      message = _ref.message,
      children = _ref.children,
      onClose = _ref.onClose,
      closeLabel = _ref.closeLabel;
  return React.createElement(Dialog, {
    title: title,
    body: message,
    footer: function footer(_ref2) {
      var close = _ref2.close;
      return React.createElement("button", {
        type: "button",
        className: "btn btn-primary",
        onClick: awaitForAsyncTask(onClose, close)
      }, closeLabel);
    }
  }, children);
}
AlertDialog.defaultProps = {
  onClose: function onClose() {},
  title: 'Atention required',
  closeLabel: 'Close'
};
AlertDialog.propTypes = {
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  message: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  onClose: PropTypes.func,
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  closeLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.element])
};

function ConfirmationDialog(_ref) {
  var title = _ref.title,
      message = _ref.message,
      children = _ref.children,
      onProceed = _ref.onProceed,
      onCancel = _ref.onCancel,
      cancelLabel = _ref.cancelLabel,
      proceedLabel = _ref.proceedLabel;
  return React.createElement(Dialog, {
    title: title,
    body: message,
    footer: function footer(_ref2) {
      var close = _ref2.close;
      return React.createElement(React.Fragment, null, React.createElement("button", {
        type: "button",
        className: "btn btn-secondary",
        onClick: awaitForAsyncTask(onCancel, close)
      }, cancelLabel), React.createElement("button", {
        type: "button",
        className: "btn btn-primary",
        onClick: awaitForAsyncTask(onProceed, close)
      }, proceedLabel));
    }
  }, children);
}
ConfirmationDialog.defaultProps = {
  onProceed: function onProceed() {},
  onCancel: function onCancel() {},
  title: 'Atention required',
  cancelLabel: 'Cancel',
  proceedLabel: 'Proceed'
};
ConfirmationDialog.propTypes = {
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  message: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  onCancel: PropTypes.func,
  onProceed: PropTypes.func,
  title: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  cancelLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  proceedLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.element])
};

var FormContext = React.createContext(null);
function validateFormElement(_ref) {
  var name = _ref.name,
      _ref$validations = _ref.validations,
      validations = _ref$validations === void 0 ? [] : _ref$validations,
      formData = _ref.formData,
      elementRefs = _ref.elementRefs;
  var validationMessage = '';
  var value = formData[name];
  validations.some(function (_ref2) {
    var message = _ref2.message,
        validate = _ref2.validate;
    var isValid = validate(value, formData);

    if (!isValid) {
      validationMessage = message;
    }

    return !isValid;
  });
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = elementRefs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var elementRef = _step.value;
      elementRef.setCustomValidity(validationMessage);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return validationMessage;
}
function handleInputChange(formState, event) {
  var target = event.target;
  var value = target.type === 'checkbox' ? target.checked : target.value;
  var name = target.name;
  formState.update(name, value);
}
function normalizeOptions(options, formData) {
  var _options = typeof options === 'function' ? options(formData) : options;

  if (!Array.isArray(_options)) {
    throw new Error('Select Options should be an array');
  }

  return _options.map(function (option) {
    if (typeof option !== 'string') {
      return option;
    }

    return {
      value: option,
      label: option
    };
  });
}

function useValueMap() {
  var _useState = useState({}),
      _useState2 = _slicedToArray(_useState, 2),
      valueMap = _useState2[0],
      updateValueMap = _useState2[1];

  function setValue(key, _value) {
    updateValueMap(function (prevValueMap) {
      var value = _value;

      if (typeof value === 'function') {
        value = value(prevValueMap[key]);
      }

      return _objectSpread2({}, prevValueMap, _defineProperty({}, key, value));
    });
  }

  function getValue(key) {
    return valueMap[key];
  }

  return {
    setValue: setValue,
    getValue: getValue,
    getAllKeys: function getAllKeys() {
      return Object.keys(valueMap);
    },
    setValueIfUnset: function setValueIfUnset(key, value) {
      if (!getValue(key)) {
        setValue(key, value);
      }
    }
  };
}
function useArrayValueMap() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$unique = _ref.unique,
      unique = _ref$unique === void 0 ? true : _ref$unique,
      _ref$equalityComparat = _ref.equalityComparator,
      equalityComparator = _ref$equalityComparat === void 0 ? function (a) {
    return function (b) {
      return a === b;
    };
  } : _ref$equalityComparat;

  var _useValueMap = useValueMap(),
      getAllKeys = _useValueMap.getAllKeys,
      getValue = _useValueMap.getValue,
      setValue = _useValueMap.setValue;

  return {
    push: function push(key, value) {
      setValue(key, function (prevValues) {
        var values = prevValues;

        if (!values) {
          values = [];
        }

        if (!unique || !values.find(equalityComparator(value))) {
          values.push(value);
        }

        return values;
      });
    },
    unset: function unset(key, filterfn) {
      setValue(key, function (prevValues) {
        return prevValues.filter(filterfn);
      });
    },
    get: getValue,
    getAllKeys: getAllKeys
  };
}

function useForm(initialState, validations) {
  var _useState = useState(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      formState = _useState2[0],
      setFormState = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      submitAttempted = _useState4[0],
      setSubmitAttempted = _useState4[1];

  var _useArrayValueMap = useArrayValueMap(),
      getElementNames = _useArrayValueMap.getAllKeys,
      getElementRefs = _useArrayValueMap.get,
      registerElementRef = _useArrayValueMap.push;

  return {
    register: function register(name, elementRef) {
      registerElementRef(name, elementRef);
      validateFormElement({
        name: name,
        formData: formState,
        validations: validations[name],
        elementRefs: [elementRef]
      });
    },
    update: function update(name, value) {
      setFormState(function (prevFormState) {
        var nextState = _objectSpread2({}, prevFormState, _defineProperty({}, name, value));

        return nextState;
      });
      this.validateForm(_objectSpread2({}, formState, _defineProperty({}, name, value)));
    },
    getFormData: function getFormData() {
      return formState;
    },
    getValue: function getValue(name) {
      return formState[name];
    },
    reset: function reset() {
      setFormState(initialState);
    },
    setSubmitedAttempted: function setSubmitedAttempted() {
      setSubmitAttempted(true);
    },
    getSubmitedAttempted: function getSubmitedAttempted() {
      return submitAttempted;
    },
    getValidationMessage: function getValidationMessage(name) {
      var elementRefs = getElementRefs(name);
      return elementRefs && elementRefs[0] ? elementRefs[0].validationMessage : '';
    },
    validateForm: function validateForm(_formData) {
      var elementNames = getElementNames();
      var isFormValid = true;
      var formData = _formData || formState;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = elementNames[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var name = _step.value;
          var isElementValid = !validateFormElement({
            name: name,
            formData: formData,
            validations: validations[name],
            elementRefs: getElementRefs(name)
          });

          if (!isElementValid) {
            isFormValid = false;
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return isFormValid;
    }
  };
}

function FormActions(_ref) {
  var submitLabel = _ref.submitLabel,
      cancelLabel = _ref.cancelLabel,
      onCancel = _ref.onCancel;
  return React.createElement("div", {
    className: "form-actions"
  }, React.createElement("button", {
    type: "submit",
    className: "btn btn-primary mr-1"
  }, submitLabel), React.createElement("button", {
    type: "button",
    className: "btn btn-secondary",
    onClick: onCancel
  }, cancelLabel));
}
FormActions.propTypes = {
  submitLabel: PropTypes.string,
  cancelLabel: PropTypes.string,
  onCancel: PropTypes.func.isRequired
};

function Form(_ref) {
  var cancelLabel = _ref.cancelLabel,
      children = _ref.children,
      customValidation = _ref.customValidation,
      initialValues = _ref.initialValues,
      onCancel = _ref.onCancel,
      onSubmit = _ref.onSubmit,
      submitLabel = _ref.submitLabel,
      validations = _ref.validations;
  var formState = useForm(initialValues, validations);
  var formRef = useRef(null);

  function resetForm() {
    formRef.current.classList.remove('was-validated');
    formState.reset();
  }

  function handleSubmit(e) {
    e.preventDefault();
    formState.setSubmitedAttempted();

    if (customValidation && !formRef.current.checkValidity()) {
      formRef.current.classList.add('was-validated');
      return;
    }

    var res = onSubmit(formState.getFormData(), resetForm);

    if (res && res.then) {
      res.then(resetForm);
    }
  }

  function handleCancel() {
    //TODO: improve cancel options
    onCancel();
  }

  var formProps = {
    ref: formRef,
    onSubmit: handleSubmit
  };

  if (customValidation) {
    formProps.noValidate = true;
  }

  return React.createElement("form", formProps, React.createElement(FormContext.Provider, {
    value: formState
  }, children), React.createElement(FormActions, {
    submitLabel: submitLabel,
    cancelLabel: cancelLabel,
    onCancel: handleCancel
  }));
}
Form.defaultProps = {
  cancelLabel: 'Cancel',
  customValidation: false,
  submitLabel: 'Submit'
};
Form.propTypes = {
  cancelLabel: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.element, PropTypes.arrayOf(PropTypes.element)]),
  customValidation: PropTypes.bool,
  initialValues: PropTypes.object,
  onCancel: PropTypes.func.isRequired,
  onSubmit: PropTypes.func.isRequired,
  submitLabel: PropTypes.string,
  validations: PropTypes.object
};

function formatClasses(cssClasses) {
  return cssClasses.filter(function (v) {
    return v;
  }).join(' ');
}

function Dropdown(_ref) {
  var children = _ref.children,
      items = _ref.items,
      onSelect = _ref.onSelect,
      isOpen = _ref.isOpen,
      onTouchStart = _ref.onTouchStart,
      onMouseEnter = _ref.onMouseEnter,
      onMouseLeave = _ref.onMouseLeave,
      template = _ref.template;
  return React.createElement("div", _extends({
    className: formatClasses(['dropdown', isOpen && 'show'])
  }, {
    onTouchStart: onTouchStart,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  }), children, items.length > 0 && React.createElement("div", {
    className: formatClasses(['dropdown-menu', isOpen && 'show']) // aria-labelledby="dropdownMenuButton"

  }, items.map(function (_ref2, index) {
    var label = _ref2.label,
        value = _ref2.value;
    return React.createElement("a", {
      key: index,
      href: "#",
      className: "dropdown-item",
      onClick: safeClick(onSelect, {
        value: value,
        index: index,
        label: label
      })
    }, template(label));
  })));
}
Dropdown.defaultProps = {
  template: function template(v) {
    return v;
  }
};
Dropdown.propTypes = {
  children: PropTypes.element,
  isOpen: PropTypes.bool,
  items: PropTypes.arrayOf(PropTypes.object),
  onMouseEnter: PropTypes.func,
  onMouseLeave: PropTypes.func,
  onSelect: PropTypes.func,
  onTouchStart: PropTypes.func,
  template: PropTypes.func
};

function FormAutocomplete(_ref) {
  var onSearch = _ref.onSearch,
      options = _ref.options,
      required = _ref.required,
      id = _ref.id,
      placeholder = _ref.placeholder,
      name = _ref.name,
      openOnFocus = _ref.openOnFocus,
      template = _ref.template,
      filter = _ref.filter;

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      searchValue = _useState2[0],
      setSearchValue = _useState2[1];

  var _useOpenState = useOpenState(),
      isOpen = _useOpenState.isOpen,
      open = _useOpenState.open,
      close = _useOpenState.close;

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      ignoreBlur = _useState4[0],
      setIgnoreBlur = _useState4[1];

  var formState = useContext(FormContext);
  var inputRef = useRef(null);
  useEffect(function () {
    formState.register(name, inputRef.current);
  }, []);
  return React.createElement(React.Fragment, null, React.createElement("input", _extends({
    required: required,
    name: name,
    id: id,
    placeholder: placeholder
  }, {
    type: "text",
    ref: inputRef,
    className: "form-control",
    onChange: handleInputChange.bind(null, {
      update: function update(_, value) {
        setSearchValue(value);
        onSearch(value);
        open();

        if (formState.getValue(name)) {
          formState.update(name, null);
        }
      }
    }),
    onFocus: function onFocus() {
      if (openOnFocus) {
        setTimeout(function () {
          open();
        }, 250);
      }
    },
    onBlur: function onBlur() {
      if (ignoreBlur) {
        inputRef.current.focus();
      } else {
        close();
      }
    },
    value: searchValue,
    role: "combobox",
    "aria-autocomplete": "list",
    "aria-expanded": "false",
    autoComplete: "off"
  })), React.createElement(Dropdown, {
    isOpen: isOpen(),
    items: normalizeOptions(options, FormData).filter(filter(searchValue)),
    onSelect: function onSelect(_ref2) {
      var value = _ref2.value,
          label = _ref2.label;
      formState.update(name, value);
      setSearchValue(label);
      close();
    },
    template: template,
    onTouchStart: function onTouchStart() {
      return setIgnoreBlur(true);
    },
    onMouseEnter: function onMouseEnter() {
      return setIgnoreBlur(true);
    },
    onMouseLeave: function onMouseLeave() {
      return setIgnoreBlur(false);
    }
  }));
}
FormAutocomplete.defaultProps = {
  openOnFocus: false,
  onSearch: function onSearch() {},
  filter: function filter(_searchValue) {
    return function (item) {
      var itemValue = JSON.stringify(item.label).toLowerCase();

      var searchValue = _searchValue.toLowerCase();

      return itemValue.includes(searchValue);
    };
  }
};
FormAutocomplete.propTypes = {
  filter: PropTypes.func,
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  onSearch: PropTypes.func,
  openOnFocus: PropTypes.bool,
  options: PropTypes.oneOfType([PropTypes.func, PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.object]))]),
  placeholder: PropTypes.string,
  required: PropTypes.any,
  template: PropTypes.func,
  type: PropTypes.string
};

function FormCheckbox(_ref) {
  var id = _ref.id,
      name = _ref.name,
      required = _ref.required,
      valueLabel = _ref.valueLabel;
  var formState = useContext(FormContext);
  var value = formState.getValue(name) || false;
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("div", {
    className: "custom-control custom-checkbox"
  }, React.createElement("input", _extends({
    required: required,
    name: name,
    id: id
  }, {
    type: "checkbox",
    className: "custom-control-input",
    onChange: handleInputChange.bind(null, formState),
    checked: value,
    ref: register
  })), React.createElement("label", {
    className: "custom-control-label",
    htmlFor: id
  }, valueLabel));
}
FormCheckbox.propTypes = {
  id: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  valueLabel: PropTypes.string,
  required: PropTypes.any
};

function FormInput(_ref) {
  var id = _ref.id,
      type = _ref.type,
      name = _ref.name,
      placeholder = _ref.placeholder,
      required = _ref.required,
      minLength = _ref.minLength,
      maxLength = _ref.maxLength,
      min = _ref.min,
      max = _ref.max,
      pattern = _ref.pattern,
      step = _ref.step;
  var formState = useContext(FormContext);
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("input", _extends({
    required: required,
    name: name,
    id: id,
    placeholder: placeholder,
    type: type,
    minLength: minLength,
    maxLength: maxLength,
    min: min,
    max: max,
    pattern: pattern,
    step: step
  }, {
    className: "form-control",
    onChange: handleInputChange.bind(null, formState),
    value: formState.getValue(name) || '',
    ref: register
  }));
}
FormInput.defaultProps = {
  type: 'text'
};
FormInput.propTypes = {
  id: PropTypes.string,
  max: PropTypes.string,
  maxLength: PropTypes.string,
  min: PropTypes.string,
  minLength: PropTypes.string,
  name: PropTypes.string.isRequired,
  pattern: PropTypes.string,
  placeholder: PropTypes.string,
  required: PropTypes.any,
  step: PropTypes.string,
  type: PropTypes.string
};

function FormLabel(_ref) {
  var id = _ref.id,
      label = _ref.label;
  return React.createElement("label", {
    htmlFor: id
  }, label);
}
FormLabel.propTypes = {
  id: PropTypes.string,
  label: PropTypes.string
};

function FormRadio(_ref) {
  var id = _ref.id,
      name = _ref.name,
      required = _ref.required,
      checkedValue = _ref.checkedValue,
      valueLabel = _ref.valueLabel,
      inline = _ref.inline;
  var formState = useContext(FormContext);
  var value = formState.getValue(name) || false;
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("div", {
    className: "custom-control custom-radio ".concat(inline ? 'custom-control-inline' : '')
  }, React.createElement("input", _extends({
    required: required,
    name: name,
    id: id
  }, {
    type: "radio",
    className: "custom-control-input",
    onChange: handleInputChange.bind(null, formState),
    checked: value === checkedValue,
    value: checkedValue,
    ref: register
  })), React.createElement("label", {
    className: "custom-control-label",
    htmlFor: id
  }, valueLabel));
}
FormRadio.defaultProps = {
  inline: false
};
FormRadio.propTypes = {
  checkedValue: PropTypes.any,
  id: PropTypes.string.isRequired,
  inline: PropTypes.bool,
  name: PropTypes.string.isRequired,
  required: PropTypes.any,
  valueLabel: PropTypes.string
};

function FormSelect(_ref) {
  var id = _ref.id,
      name = _ref.name,
      options = _ref.options,
      required = _ref.required,
      placeholder = _ref.placeholder;
  var formState = useContext(FormContext);
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("select", _extends({
    required: required,
    name: name,
    id: id
  }, {
    className: "custom-select",
    onChange: handleInputChange.bind(null, formState),
    value: formState.getValue(name) || '',
    ref: register
  }), React.createElement("option", {
    value: ""
  }, placeholder), renderOptions(options, formState.getFormData()));
}
FormSelect.propTypes = {
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  options: PropTypes.oneOfType([PropTypes.func, PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.object]))]),
  placeholder: PropTypes.string,
  required: PropTypes.any
};

function renderOptions(options, formData) {
  return normalizeOptions(options, formData).map(function (_ref2, index) {
    var value = _ref2.value,
        label = _ref2.label;
    return React.createElement("option", {
      key: index,
      value: value
    }, label);
  });
}

function FormSwitch(_ref) {
  var id = _ref.id,
      name = _ref.name,
      required = _ref.required,
      trueLabel = _ref.trueLabel,
      falseLabel = _ref.falseLabel;
  var formState = useContext(FormContext);
  var value = formState.getValue(name) || false;
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("div", {
    className: "custom-control custom-switch"
  }, React.createElement("input", _extends({
    required: required,
    name: name,
    id: id
  }, {
    type: "checkbox",
    className: "custom-control-input",
    onChange: handleInputChange.bind(null, formState),
    checked: value,
    ref: register
  })), React.createElement("label", {
    className: "custom-control-label",
    htmlFor: id
  }, (trueLabel || falseLabel) && (value ? trueLabel : falseLabel)));
}
FormSwitch.propTypes = {
  falseLabel: PropTypes.string,
  id: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  required: PropTypes.any,
  trueLabel: PropTypes.string
};

function FormTextarea(_ref) {
  var id = _ref.id,
      name = _ref.name,
      required = _ref.required,
      placeholder = _ref.placeholder,
      rows = _ref.rows;
  var formState = useContext(FormContext);
  var register = useCallback(function (ref) {
    formState.register(name, ref);
  }, []);
  return React.createElement("textarea", _extends({
    required: required,
    name: name,
    id: id,
    placeholder: placeholder,
    rows: rows
  }, {
    className: "form-control",
    onChange: handleInputChange.bind(null, formState),
    value: formState.getValue(name) || '',
    ref: register
  }));
}
FormTextarea.defaultProps = {
  rows: 5
};
FormTextarea.propTypes = {
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  placeholder: PropTypes.string,
  required: PropTypes.any,
  rows: PropTypes.number
};

function FormValidationFeedback(_ref) {
  var name = _ref.name,
      mockInvalidSibling = _ref.mockInvalidSibling;
  var formState = useContext(FormContext);
  var validationMessage = formState.getValidationMessage(name);
  return React.createElement(React.Fragment, null, mockInvalidSibling && formState.getSubmitedAttempted() && validationMessage && React.createElement("div", {
    className: "is-invalid"
  }), React.createElement("div", {
    className: validationMessage ? 'invalid-feedback' : 'valid-feedback'
  }, validationMessage || React.createElement("span", null, "\xA0")));
}
FormValidationFeedback.defaultProps = {
  mockInvalidSibling: false
};
FormValidationFeedback.propTypes = {
  name: PropTypes.string.isRequired,
  mockInvalidSibling: PropTypes.bool
};

function FormGroup(_ref) {
  var children = _ref.children,
      name = _ref.name,
      feedback = _ref.feedback,
      mockInvalidSibling = _ref.mockInvalidSibling,
      props = _objectWithoutProperties(_ref, ["children", "name", "feedback", "mockInvalidSibling"]);

  return React.createElement("div", {
    className: "form-group"
  }, React.createElement(FormLabel, props), children, feedback && React.createElement(FormValidationFeedback, {
    mockInvalidSibling: mockInvalidSibling,
    name: name
  }));
}

FormGroup.defaultProps = {
  feedback: true,
  mockInvalidSibling: false
};
FormGroup.propTypes = {
  children: PropTypes.element,
  feedback: PropTypes.bool,
  mockInvalidSibling: PropTypes.bool,
  name: PropTypes.string.isRequired
};
function FormGroupAutocomplete(props) {
  return React.createElement(FormGroup, props, React.createElement(FormAutocomplete, props));
}
function FormGroupCheckbox(props) {
  return React.createElement(FormGroup, _extends({
    mockInvalidSibling: true
  }, props), React.createElement(FormCheckbox, props));
}
function FormGroupInput(props) {
  return React.createElement(FormGroup, props, React.createElement(FormInput, props));
}
function FormGroupRadio(_ref2) {
  var options = _ref2.options,
      id = _ref2.id,
      props = _objectWithoutProperties(_ref2, ["options", "id"]);

  return React.createElement(FormGroup, _extends({
    mockInvalidSibling: true
  }, props), React.createElement("div", null, options.map(function (option, index) {
    return React.createElement(FormRadio, _extends({
      key: index
    }, props, {
      checkedValue: option.value,
      valueLabel: option.label,
      id: "".concat(id, "-").concat(index)
    }));
  })));
}
FormGroupRadio.defaultProps = {
  inline: true
};
FormGroupRadio.propTypes = {
  id: PropTypes.string,
  options: PropTypes.arrayOf(PropTypes.object),
  inline: PropTypes.bool
};
function FormGroupSelect(props) {
  return React.createElement(FormGroup, props, React.createElement(FormSelect, props));
}
function FormGroupSwitch(props) {
  return React.createElement(FormGroup, _extends({
    mockInvalidSibling: true
  }, props), React.createElement(FormSwitch, props));
}
function FormGroupTextarea(props) {
  return React.createElement(FormGroup, props, React.createElement(FormTextarea, props));
}

function ListGroupItem(_ref) {
  var index = _ref.index,
      isActive = _ref.isActive,
      children = _ref.children,
      linked = _ref.linked,
      isDisabled = _ref.isDisabled,
      onSelect = _ref.onSelect;
  var classes = ['list-group-item', isActive && 'active', isDisabled && 'disabled', linked && 'list-group-item-action'].filter(function (v) {
    return v;
  }).join(' ');

  if (linked) {
    return React.createElement("a", {
      href: "#",
      className: classes,
      onClick: safeClick(onSelect, index)
    }, children);
  }

  return React.createElement("li", {
    className: classes,
    onClick: safeClick(onSelect, index)
  }, children);
}
ListGroupItem.defaultProps = {
  isActive: false
};
ListGroupItem.propTypes = {
  children: PropTypes.element.isRequired,
  index: PropTypes.number.isRequired,
  isActive: PropTypes.bool,
  isDisabled: PropTypes.bool,
  linked: PropTypes.bool.isRequired,
  onSelect: PropTypes.func
};

function ListGroup(_ref) {
  var items = _ref.items,
      active = _ref.active,
      linked = _ref.linked,
      bordered = _ref.bordered,
      horizontal = _ref.horizontal,
      template = _ref.template,
      isDisabled = _ref.isDisabled,
      onSelect = _ref.onSelect;
  var content = items.map(function (item, index) {
    return React.createElement(ListGroupItem, {
      key: index,
      index: index,
      isActive: active === index,
      isDisabled: isDisabled(item),
      item: item,
      linked: linked,
      onSelect: onSelect
    }, template(item));
  });
  var classes = ['list-group', !bordered && 'list-group-flush', horizontal && 'list-group-horizontal'].filter(function (v) {
    return v;
  }).join(' ');

  if (linked) {
    return React.createElement("div", {
      className: classes
    }, content);
  }

  return React.createElement("ul", {
    className: classes
  }, content);
}
ListGroup.defaultProps = {
  bordered: true,
  horizontal: false,
  isDisabled: function isDisabled() {},
  linked: false,
  onSelect: function onSelect() {}
};
ListGroup.propTypes = {
  active: PropTypes.number,
  bordered: PropTypes.bool,
  horizontal: PropTypes.bool,
  isDisabled: PropTypes.func,
  items: PropTypes.array.isRequired,
  linked: PropTypes.bool,
  onSelect: PropTypes.func,
  template: PropTypes.func.isRequired
};

function useSelectedItem(defaultIndex, collectionSize) {
  var _useState = useState(defaultIndex < collectionSize ? defaultIndex : 0),
      _useState2 = _slicedToArray(_useState, 2),
      selectedItem = _useState2[0],
      setSelectedItem = _useState2[1];

  return {
    getSelected: function getSelected() {
      return selectedItem;
    },
    select: function select(index) {
      if (index >= collectionSize) {
        throw new Error('Invalid tab');
      }

      setSelectedItem(index);
    }
  };
}

function StatefulListGroup(_ref) {
  var initialItem = _ref.initialItem,
      items = _ref.items,
      _ = _ref.active,
      props = _objectWithoutProperties(_ref, ["initialItem", "items", "active"]);

  var _useSelectedItem = useSelectedItem(initialItem, items.length),
      getSelected = _useSelectedItem.getSelected,
      select = _useSelectedItem.select;
  return React.createElement(ListGroup, _extends({
    items: items,
    active: getSelected(),
    onSelect: select
  }, props));
}
StatefulListGroup.propTypes = {
  active: PropTypes.any,
  items: PropTypes.arrayOf(PropTypes.object),
  initialItem: PropTypes.number
};

function Pagination(_ref) {
  var actualPage = _ref.actualPage,
      ariaLabel = _ref.ariaLabel,
      firstLabel = _ref.firstLabel,
      lastLabel = _ref.lastLabel,
      lastPage = _ref.lastPage,
      maxPageWindow = _ref.maxPageWindow,
      nextLabel = _ref.nextLabel,
      onSelect = _ref.onSelect,
      previousLabel = _ref.previousLabel;
  var pageWindow = Math.min(maxPageWindow, lastPage);
  var centerOfPageWindow = Math.max(Math.floor(pageWindow / 2), 2);
  var lastStartPage = Math.max(lastPage - pageWindow + 1, 1);
  var startPage = Math.max(Math.min(actualPage - centerOfPageWindow, lastStartPage), 1);
  var showPreviousNavigation = startPage > 1;
  var pages = Array.from({
    length: pageWindow
  }, function (v, i) {
    return startPage + i;
  });

  var select = function select(page) {
    return safeClick(onSelect, page);
  };

  return React.createElement("nav", {
    "aria-label": ariaLabel
  }, React.createElement("ul", {
    className: "pagination"
  }, showPreviousNavigation && React.createElement(React.Fragment, null, React.createElement("li", {
    className: "page-item"
  }, React.createElement("a", {
    className: "page-link",
    href: "",
    onClick: select(1),
    "aria-label": firstLabel
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\xAB"), React.createElement("span", {
    className: "sr-only"
  }, firstLabel))), React.createElement("li", {
    className: "page-item"
  }, React.createElement("a", {
    className: "page-link",
    href: "",
    onClick: select(actualPage - 1),
    "aria-label": previousLabel
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\u2039"), React.createElement("span", {
    className: "sr-only"
  }, previousLabel)))), pages.map(function (page) {
    return React.createElement("li", {
      key: "pages-".concat(page),
      className: "page-item".concat(actualPage === page ? ' active' : '')
    }, React.createElement("a", {
      className: "page-link",
      href: "",
      onClick: select(page)
    }, page));
  }), startPage < lastStartPage && React.createElement(React.Fragment, null, React.createElement("li", {
    className: "page-item"
  }, React.createElement("a", {
    className: "page-link",
    href: "",
    onClick: select(actualPage + 1),
    "aria-label": nextLabel
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\u203A"), React.createElement("span", {
    className: "sr-only"
  }, nextLabel))), React.createElement("li", {
    className: "page-item"
  }, React.createElement("a", {
    className: "page-link",
    href: "",
    onClick: select(lastPage),
    "aria-label": lastLabel
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\xBB"), React.createElement("span", {
    className: "sr-only"
  }, lastLabel))))));
}
Pagination.defaultProps = {
  actualPage: 1,
  ariaLabel: 'Page navigation',
  firstLabel: 'First',
  lastLabel: 'Last',
  maxPageWindow: 5,
  nextLabel: 'Next',
  previousLabel: 'Previous'
};
Pagination.propTypes = {
  actualPage: PropTypes.number,
  ariaLabel: PropTypes.string,
  firstLabel: PropTypes.string,
  lastLabel: PropTypes.string,
  lastPage: PropTypes.number,
  maxPageWindow: PropTypes.number,
  nextLabel: PropTypes.string,
  onSelect: PropTypes.func.isRequired,
  previousLabel: PropTypes.string
};

function normalizeColumns(columns) {
  return columns.map(function (column) {
    if (typeof column !== 'string') {
      return column;
    }

    return {
      attribute: column,
      label: column
    };
  });
}
function getColumnClass(_ref) {
  var align = _ref.align;
  var classes = [align === 'center' ? 'text-center' : align === 'right' ? 'text-right' : ''];
  return classes.join(' ');
}

function TableHead(_ref) {
  var columns = _ref.columns,
      actions = _ref.actions,
      actionLabel = _ref.actionLabel;
  return React.createElement("thead", null, React.createElement("tr", null, columns.map(function (column, columnIndex) {
    return React.createElement("th", {
      key: columnIndex,
      className: getColumnClass(column)
    }, column.label);
  }), actions && React.createElement("th", {
    className: "text-center"
  }, actionLabel)));
}
TableHead.propTypes = {
  actionLabel: PropTypes.string,
  actions: PropTypes.arrayOf(PropTypes.object),
  columns: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.object]))
};

function TableBody(_ref) {
  var columns = _ref.columns,
      docs = _ref.docs,
      rowClass = _ref.rowClass,
      actions = _ref.actions;
  return React.createElement("tbody", null, docs.map(function (doc, docIndex) {
    return React.createElement("tr", {
      key: docIndex,
      className: rowClass(doc)
    }, columns.map(function (column, columnIndex) {
      return React.createElement("td", {
        key: columnIndex,
        className: getColumnClass(column)
      }, getColumnValue(doc, column, docIndex));
    }), actions && React.createElement("td", {
      className: "text-center"
    }, actions.map(function (action, actionIndex) {
      return React.createElement("a", _extends({
        key: actionIndex,
        title: action.title
      }, getActionProps(action, doc), {
        className: actionIndex > 0 ? 'ml-2' : ''
      }), action.content);
    })));
  }));
}
TableBody.propTypes = {
  actions: PropTypes.arrayOf(PropTypes.object),
  columns: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.object])),
  docs: PropTypes.arrayOf(PropTypes.object),
  rowClass: PropTypes.func
};

function getColumnValue(doc, column, docIndex) {
  var rawValue = doc[column.attribute];

  if (!column.format) {
    return rawValue;
  }

  return column.format(rawValue, doc, docIndex);
}

function getActionProps(action, doc) {
  var props = {};

  if (action.onClick) {
    props.href = '';
    props.onClick = safeClick(action.onClick, doc);
  } else if (action.link) {
    props.href = action.link(doc);
  }

  return props;
}

function Table(_ref) {
  var docs = _ref.docs,
      columns = _ref.columns,
      striped = _ref.striped,
      bordered = _ref.bordered,
      hover = _ref.hover,
      small = _ref.small,
      dark = _ref.dark,
      rowClass = _ref.rowClass,
      caption = _ref.caption,
      actions = _ref.actions,
      actionLabel = _ref.actionLabel;
  var normalizedColumns = normalizeColumns(columns);
  var tableClasses = ['table', striped && 'table-striped', bordered && 'table-bordered', hover && 'table-hover', small && 'table-sm', dark && 'table-dark'].join(' ');
  return React.createElement("div", {
    className: "table-responsive"
  }, React.createElement("table", {
    className: tableClasses
  }, caption && React.createElement("caption", null, caption), React.createElement(TableHead, _extends({
    actions: actions,
    actionLabel: actionLabel
  }, {
    columns: normalizedColumns
  })), React.createElement(TableBody, _extends({
    docs: docs,
    rowClass: rowClass,
    actions: actions
  }, {
    columns: normalizedColumns
  }))));
}
Table.defaultProps = {
  striped: true,
  bordered: false,
  hover: true,
  small: true,
  dark: false,
  actionLabel: 'Actions',
  rowClass: function rowClass() {
    return '';
  }
};
Table.propTypes = {
  actions: PropTypes.arrayOf(PropTypes.object),
  columns: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.object])),
  docs: PropTypes.arrayOf(PropTypes.object),
  rowClass: PropTypes.func,
  striped: PropTypes.bool,
  bordered: PropTypes.bool,
  hover: PropTypes.bool,
  small: PropTypes.bool,
  dark: PropTypes.bool,
  caption: PropTypes.oneOfType([PropTypes.string, PropTypes.element]),
  actionLabel: PropTypes.string
};

function TabHeader(_ref) {
  var index = _ref.index,
      isActive = _ref.isActive,
      title = _ref.title,
      onSelect = _ref.onSelect;
  return React.createElement("li", {
    className: "nav-item"
  }, React.createElement("a", {
    className: "nav-link ".concat(isActive ? 'active' : '') // id="home-tab"
    ,
    "data-toggle": "tab",
    href: "" // href="#home"
    ,
    onClick: safeClick(onSelect, index),
    role: "tab" // aria-controls="home"
    ,
    "aria-selected": isActive
  }, title));
}
TabHeader.propTypes = {
  index: PropTypes.number,
  isActive: PropTypes.bool,
  title: PropTypes.string,
  onSelect: PropTypes.func
};

function TabContent(_ref) {
  var isActive = _ref.isActive,
      content = _ref.content;
  return React.createElement("div", {
    className: "tab-pane fade ".concat(isActive ? 'show active' : '') // id="home"
    ,
    role: "tabpanel" // aria-labelledby="home-tab"

  }, content);
}
TabContent.propTypes = {
  isActive: PropTypes.bool,
  content: PropTypes.oneOfType([PropTypes.element, PropTypes.string])
};

function Tabs(_ref) {
  var vertical = _ref.vertical,
      tabs = _ref.tabs,
      activeTab = _ref.activeTab,
      onlyRenderActiveTab = _ref.onlyRenderActiveTab,
      bordered = _ref.bordered,
      onSelect = _ref.onSelect;

  if (activeTab >= tabs.length) {
    console.warn('Invalid tab selected:', activeTab);
  }

  return React.createElement("div", {
    className: "custom-tabs-container ".concat(vertical ? 'd-flex' : '')
  }, React.createElement("div", {
    className: "tabs-navigation"
  }, React.createElement("ul", {
    className: "nav ".concat(vertical ? 'nav-pills flex-column' : 'nav-tabs'),
    id: "myTab",
    role: "tablist"
  }, tabs.map(function (tab, tabIndex) {
    return React.createElement(TabHeader, {
      key: tabIndex,
      index: tabIndex,
      isActive: tabIndex === activeTab,
      title: tab.title,
      onSelect: onSelect
    });
  }))), React.createElement("div", {
    className: "tab-content ".concat(vertical ? 'flex-fill ml-3' : "".concat(bordered ? 'border-left border-right border-bottom p-2' : 'py-2')) // id="myTabContent"

  }, onlyRenderActiveTab ? React.createElement(TabContent, {
    isActive: true,
    content: tabs[activeTab] && tabs[activeTab].content
  }) : tabs.map(function (tab, tabIndex) {
    return React.createElement(TabContent, {
      key: tabIndex,
      isActive: tabIndex === activeTab,
      content: tab.content
    });
  })));
}
Tabs.defaultProps = {
  vertical: false,
  bordered: false,
  activeTab: 0,
  onlyRenderActiveTab: false,
  onSelect: function onSelect() {}
};
Tabs.propTypes = {
  vertical: PropTypes.bool,
  tabs: PropTypes.arrayOf(PropTypes.object),
  activeTab: PropTypes.number,
  onlyRenderActiveTab: PropTypes.bool,
  bordered: PropTypes.bool,
  onSelect: PropTypes.func
};

function StatefulTabs(_ref) {
  var initialTab = _ref.initialTab,
      _ = _ref.activeTab,
      tabs = _ref.tabs,
      props = _objectWithoutProperties(_ref, ["initialTab", "activeTab", "tabs"]);

  var _useSelectedItem = useSelectedItem(initialTab, tabs.length),
      getSelected = _useSelectedItem.getSelected,
      select = _useSelectedItem.select;
  return React.createElement(Tabs, _extends({
    tabs: tabs
  }, props, {
    activeTab: getSelected(),
    onSelect: select
  }));
}
StatefulTabs.propTypes = {
  activeTab: PropTypes.any,
  tabs: PropTypes.arrayOf(PropTypes.object),
  initialTab: PropTypes.number
};

var ToastsContext = React.createContext(null);
var toastPositions = [{
  name: 'TOP_LEFT',
  className: 'top-left'
}, {
  name: 'TOP_RIGHT',
  className: 'top-right'
}, {
  name: 'BOTTOM_LEFT',
  className: 'bottom-left'
}, {
  name: 'BOTTOM_RIGHT',
  className: 'bottom-right'
}];
var toastsDefaultStylesByPosition = {
  TOP_RIGHT: {
    top: '25px',
    right: '25px'
  },
  TOP_LEFT: {
    top: '25px',
    left: '25px'
  },
  BOTTOM_RIGHT: {
    right: '25px',
    bottom: '25px'
  },
  BOTTOM_LEFT: {
    left: '25px',
    bottom: '25px'
  }
};

function Toast(_ref) {
  var id = _ref.id,
      type = _ref.type,
      message = _ref.message,
      closeControl = _ref.closeControl,
      position = _ref.position,
      noStyle = _ref.noStyle;
  var toastsState = useContext(ToastsContext);
  return React.createElement("div", {
    className: formatClasses(['toast-message', 'alert', "alert-".concat(type), closeControl && 'alert-dismissible']),
    style: noStyle ? null : {
      width: 'auto',
      zIndex: 9999
    }
  }, message, closeControl && React.createElement("button", {
    type: "button",
    className: "close",
    onClick: function onClick() {
      return toastsState.close(position, id);
    },
    "aria-label": "Close"
  }, React.createElement("span", {
    "aria-hidden": "true"
  }, "\xD7")));
}
Toast.propTypes = {
  closeControl: PropTypes.bool,
  id: PropTypes.number.isRequired,
  message: PropTypes.string.isRequired,
  noStyle: PropTypes.bool,
  position: PropTypes.string.isRequired,
  type: PropTypes.string.isRequired
};

function ToastsRegion(_ref) {
  var name = _ref.name,
      className = _ref.className,
      toasts = _ref.toasts,
      noStyle = _ref.noStyle;
  return React.createElement("div", {
    key: name,
    className: className,
    style: noStyle ? null : _objectSpread2({
      position: 'fixed',
      zIndex: 9999,
      maxWidth: '50%'
    }, toastsDefaultStylesByPosition[name])
  }, toasts.map(function (toast) {
    return React.createElement(Toast, _extends({
      key: toast.id
    }, toast));
  }));
}
ToastsRegion.defaultProps = {
  toasts: []
};
ToastsRegion.propTypes = {
  className: PropTypes.string.isRequired,
  name: PropTypes.string.isRequired,
  noStyle: PropTypes.bool,
  toasts: PropTypes.arrayOf(PropTypes.object)
};

function useToast(_ref) {
  var unique = _ref.unique;

  var _useState = useState(0),
      _useState2 = _slicedToArray(_useState, 2),
      nextId = _useState2[0],
      setNextId = _useState2[1];

  var _useArrayValueMap = useArrayValueMap(unique && {
    equalityComparator: function equalityComparator(a) {
      return function (b) {
        return a.message === b.message;
      };
    }
  }),
      push = _useArrayValueMap.push,
      unset = _useArrayValueMap.unset,
      get = _useArrayValueMap.get;

  function close(position, toastId) {
    unset(position, function (toast) {
      return toast.id !== toastId;
    });
  }

  return {
    show: function show(message) {
      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref2$type = _ref2.type,
          type = _ref2$type === void 0 ? 'info' : _ref2$type,
          _ref2$autoClose = _ref2.autoClose,
          autoClose = _ref2$autoClose === void 0 ? 5000 : _ref2$autoClose,
          _ref2$position = _ref2.position,
          position = _ref2$position === void 0 ? 'TOP_RIGHT' : _ref2$position;

      if (!['info', 'success', 'danger', 'warning'].includes(type)) {
        throw new Error("Invalid toast type ".concat(type, ". Must be 'info', 'success', 'danger' or 'warning'"));
      }

      var toastId = nextId;
      push(position, {
        id: toastId,
        message: message,
        type: type,
        position: position,
        closeControl: !autoClose
      });

      if (autoClose && typeof autoClose === 'number' && !isNaN(autoClose)) {
        setTimeout(function () {
          close(position, toastId);
        }, autoClose);
      }

      setNextId(function (prevId) {
        return prevId + 1;
      });
    },
    close: close,
    get: get
  };
}

function ToastsContainer(_ref) {
  var children = _ref.children,
      unique = _ref.unique,
      noStyle = _ref.noStyle;
  var toastsState = useToast({
    unique: unique
  });
  return React.createElement(ToastsContext.Provider, {
    value: toastsState
  }, children, React.createElement("div", {
    className: "toast-container"
  }, toastPositions.map(function (_ref2) {
    var name = _ref2.name,
        className = _ref2.className;
    return React.createElement(ToastsRegion, {
      key: name,
      name: name,
      className: className,
      toasts: toastsState.get(name),
      noStyle: noStyle
    });
  })));
}
ToastsContainer.defaultProps = {
  noStyle: false,
  unique: true
};
ToastsContainer.propTypes = {
  children: PropTypes.element,
  noStyle: PropTypes.bool,
  unique: PropTypes.bool
};

export { AlertDialog, ConfirmationDialog, Dialog, Dropdown, Form, FormAutocomplete, FormCheckbox, FormGroupAutocomplete, FormGroupCheckbox, FormGroupInput, FormGroupRadio, FormGroupSelect, FormGroupSwitch, FormGroupTextarea, FormInput, FormRadio, FormSelect, FormSwitch, FormTextarea, ListGroup, Pagination, StatefulListGroup, StatefulTabs, TabContent, TabHeader, Table, Tabs, ToastsContainer, ToastsContext, toastPositions, toastsDefaultStylesByPosition };
